/**************************************************************************
 *
 * Copyright 2016 Samsung Electronics co., Ltd. All Rights Reserved.
 *
 * Contact: Konstantin Drabeniuk <k.drabeniuk@samsung.com>
 * Contact: Andrii Sokolenko <a.sokolenko@samsung.com>
 * Contact: Roman Marchenko <r.marchenko@samsung.com>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sub license, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * The above copyright notice and this permission notice (including the
 * next paragraph) shall be included in all copies or substantial portions
 * of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT.
 * IN NO EVENT SHALL PRECISION INSIGHT AND/OR ITS SUPPLIERS BE LIABLE FOR
 * ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
**************************************************************************/

#ifndef UT_H
#define UT_H

#include <vector>
#include <list>
#include <climits>
#include <pthread.h>
#include <gmock/gmock.h>
#include <stdlib.h>
#include <tbm_bufmgr_int.h>
#include <wayland-server-protocol.h>
#include <tbm_log.h>

#undef TBM_DBG
#undef TBM_INFO
#undef TBM_WRN
#undef TBM_ERR
#define TBM_DBG(fmt, args...)  tbm_log_print(TBM_LOG_LEVEL_DBG, fmt, ##args)
#define TBM_INFO(fmt, args...) tbm_log_print(TBM_LOG_LEVEL_INFO, fmt, ##args)
#define TBM_WRN(fmt, args...)  tbm_log_print(TBM_LOG_LEVEL_WRN, fmt, ##args)
#define TBM_ERR(fmt, args...)  tbm_log_print(TBM_LOG_LEVEL_ERR, fmt, ##args)

#define TBM_UT_INFO(fmt, args...) tbm_log_print_stdout(TBM_LOG_LEVEL_INFO, fmt, ##args)
#define TBM_UT_WRN(fmt, args...)  tbm_log_print_stdout(TBM_LOG_LEVEL_WRN, fmt, ##args)
#define TBM_UT_ERR(fmt, args...)  tbm_log_print_stdout(TBM_LOG_LEVEL_ERR, fmt, ##args)

#define TBM_UT_ENTRY() \
    TBM_INFO("--------------------------------------------- %s", typeid(*this).name())

#define TBM_UT_CHECK_FLAG(FLAG) \
	do {\
		if(!(FLAG)) \
			TBM_UT_INFO("     [          ] not supported");\
			EXPECT_TRUE(0);\
	} while(0)

#define TBM_UT_RETURN_TRUE_NOT_SUPPORTED(error) \
	do {\
		if((error == TBM_ERROR_NOT_SUPPORTED)) {\
			TBM_UT_WRN("     [  SKIPPED ] not supported");\
			EXPECT_TRUE(1);\
			return;\
		}\
	} while(0)

#define TBM_UT_NEVER_GET_HERE() \
	do {\
		TBM_UT_INFO("!!! TBM UT NEVER GET HERE !!!");\
	} while(0)
#define TBM_UT_RETURN_IF_FAIL(cond) { \
	if (!(cond)) { \
		TBM_UT_ERR("[%s,%d] '%s' failed", __FUNCTION__, __LINE__, #cond); \
		return; \
	} \
}
#define TBM_UT_RETURN_FALSE_IF_FAIL(cond) { \
	if (!(cond)) { \
		TBM_UT_ERR("[%s,%d] '%s' failed", __FUNCTION__, __LINE__, #cond); \
		return false; \
	} \
}
#define TBM_UT_GOTO_IF_FAIL(cond, dst) { \
	if (!(cond)) { \
		TBM_UT_ERR("[%s,%d] '%s' failed", __FUNCTION__, __LINE__, #cond); \
		goto dst; \
	} \
}

#define TBM_UT_DUMP_DIR            "/tmp/tbm_dump"
#define TBM_UT_INVALID_VALUE       -42
#define TBM_UT_INVALID_UINT_VALUE  99999
#define TBM_UT_BO_SIZE             1024
#define TBM_UT_SURFACE_QUEUE_SIZE  4

using ::testing::TestWithParam;
using ::testing::Bool;
using ::testing::Values;
using ::testing::Combine;

class TBMEnv : public TestWithParam< std::tuple<bool, bool, const char*> >
{
public:
	void SetUp(void);
	void TearDown(void);
};

class TBMBufmgr : public TBMEnv
{
public:
	TBMBufmgr();
	void SetUp(void);
	void TearDown(void);

	tbm_bufmgr bufmgr;
};

#endif // UT_H
