/**************************************************************************

libtbm

Copyright 2018 Samsung Electronics co., Ltd. All Rights Reserved.

Contact: SooChan Lim <sc1.lim@samsung.com>,
		 Sangjin Lee <lsj119@samsung.com>,
		 Boram Park <boram1288.park@samsung.com>,
		 Changyeon Lee <cyeon.lee@samsung.com>

Permission is hereby granted, free of charge, to any person obtaining a
copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including
without limitation the rights to use, copy, modify, merge, publish,
distribute, sub license, and/or sell copies of the Software, and to
permit persons to whom the Software is furnished to do so, subject to
the following conditions:

The above copyright notice and this permission notice (including the
next paragraph) shall be included in all copies or substantial portions
of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT.
IN NO EVENT SHALL PRECISION INSIGHT AND/OR ITS SUPPLIERS BE LIABLE FOR
ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

**************************************************************************/

#include "config.h"
#include "tbm_bufmgr_int.h"

/* LCOV_EXCL_START */
int
tbm_backend_bufmgr_query_display_server(tbm_bufmgr bufmgr, tbm_error_e *error)
{
	const char *value;

	if (!bufmgr) {
		if (error)
			*error = TBM_ERROR_INVALID_PARAMETER;
		return 0;
	}

	/* TODO: TBM_DISPLAY_SERVER will be removed */
	value = (const char*)getenv("TBM_DISPLAY_SERVER");

	if (value || gBufMgr->display_server) {
		if (error)
			*error = TBM_ERROR_NONE;
		return 1;
	}

	if (error)
		*error = TBM_ERROR_NONE;

	return 0;
}

tbm_backend_bufmgr_func *
tbm_backend_bufmgr_alloc_bufmgr_func(tbm_bufmgr bufmgr, tbm_error_e *error)
{
	tbm_backend_bufmgr_func *bufmgr_func;

	bufmgr_func = calloc(1, sizeof(struct _tbm_backend_bufmgr_func));
	if (!bufmgr_func) {
		TBM_ERR("error: fail to allocate the tbm_backend_bufmgr_func\n");
		if (error)
			*error = TBM_ERROR_OUT_OF_MEMORY;
		return NULL;
	}

	if (error)
		*error = TBM_ERROR_NONE;

	return bufmgr_func;
}

void
tbm_backend_bufmgr_free_bufmgr_func(tbm_bufmgr bufmgr, tbm_backend_bufmgr_func *func)
{
	TBM_RETURN_IF_FAIL(bufmgr);

	if (func)
		free(func);
}

tbm_error_e
tbm_backend_bufmgr_register_bufmgr_func(tbm_bufmgr bufmgr, tbm_backend_bufmgr_func *func)
{
	TBM_RETURN_VAL_IF_FAIL(bufmgr, TBM_ERROR_INVALID_PARAMETER);
	TBM_RETURN_VAL_IF_FAIL(func, TBM_ERROR_INVALID_PARAMETER);

	// The tbm_bufmgr from backend module is actually tbm_module memory in libtbm.
	// libtbm initializes module init with this tbm_module which is casted with tbm_bufmgr.
	tbm_module *module = (tbm_module *)bufmgr;
	module->bufmgr_func = func;

	return TBM_ERROR_NONE;
}

tbm_backend_bo_func *
tbm_backend_bufmgr_alloc_bo_func(tbm_bufmgr bufmgr, tbm_error_e *error)
{
	tbm_backend_bo_func *bufmgr_bo;

	bufmgr_bo = calloc(1, sizeof(struct _tbm_backend_bo_func));
	if (!bufmgr_bo) {
		TBM_ERR("error: fail to allocate the tbm_backend_bo_func\n");
		if (error)
			*error = TBM_ERROR_OUT_OF_MEMORY;
		return NULL;
	}

	if (error)
		*error = TBM_ERROR_NONE;

	return bufmgr_bo;
}

void
tbm_backend_bufmgr_free_bo_func(tbm_bufmgr bufmgr, tbm_backend_bo_func *func)
{
	TBM_RETURN_IF_FAIL(bufmgr);

	if (func)
		free(func);
}

tbm_error_e
tbm_backend_bufmgr_register_bo_func(tbm_bufmgr bufmgr, tbm_backend_bo_func *func)
{
	TBM_RETURN_VAL_IF_FAIL(bufmgr, TBM_ERROR_INVALID_PARAMETER);
	TBM_RETURN_VAL_IF_FAIL(func, TBM_ERROR_INVALID_PARAMETER);

	// The tbm_bufmgr from backend module is actually tbm_module memory in libtbm.
	// libtbm initializes module init with this tbm_module which is casted with tbm_bufmgr.
	tbm_module *module = (tbm_module *)bufmgr;
	module->bo_func = func;

	return TBM_ERROR_NONE;
}

/* LCOV_EXCL_STOP */
